////////////////////////////////////////////////////////////////////////////
//
//  CODEC_INTERFACE.PAS
//
//  Stellt zwei Klassen zur Konfiguration der Codecs zur Verfgung. Eine Klasse
//  stellt eine Abstrahierung eines SPI-Ports (ber GPO-Leitungen) dar. Mit dieser
//  Klasse lassen sich Port-Toggle-Befehle zu einem String zusammenbauen, welche
//  dann an den FPGA gesendet werden.
//  Die zweite Klasse implementiert das Senden von SPI-Befehlen und ermglicht
//  das setzen der Register fr den TLV320AIC23B-Audio-Codec.
//
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit codec_interface;

interface

uses  windows, sysutils, classes, math,

      fpga_com_base;


type  // Stellt port-toggle befehle zur verfgung die in einem String zusammengebaut
      // werden. Dieser String wird per WriteGPONibbleBuffer in der FPGACOMBase-Klasse
      // versandt.

      TCodecInterfaceBuilder = class
        temp : ansistring;      // string der aus den Port-Toggle befehlen besteht
        cs1, cs2, spi_clk, spi_dout : byte; // spi-port-pins zustand

        procedure Append;       // aktuelles nibble an string anhngen

      public
        procedure NewString;    // neuen string zusammenbauen

        procedure Nop;          // NOP (einfach ein Append)

        procedure SetCS1;       // setzt/lscht Chip-Select Leitung 1
        procedure ClrCS1;

        procedure SetCS2;       // setzt/lscht Chip-Select Leitung 2
        procedure ClrCS2;

        procedure SetSPIClk;    // setzt/lscht SPI-Clock Leitung
        procedure ClrSPIClk;

        procedure SetSPIDout;   // setzt/lscht SPI-Datenleitung
        procedure ClrSPIDout;

        function  GetString : ansistring; // liest aktuellen string aus und setzt temp zurck

        constructor create;
      end;

      // diese klasse setzt die spi-port-toggle befehle zu einer spi-sendeeinheit
      // zusammen. weiterhin werden registerfunktionen der Codecs, sowie
      // das senden des zusammengesetzen strings bereitgestellt.

      TCodecInterface = class
      private
        cib       : TCodecInterfaceBuilder;  // port-toggle klasse
        ffpgacom  : TFPGAComBase;            // verbindung zum senden des port-togglens
        fchip     : byte;                    // codec-nummer
        fregs     : array [0 .. 9] of word;  // interner status aller register

        procedure InitData;                  // daten anhand der chip-nummer initialisieren
        procedure SendAll;                   // codec resetten und alle register setzen

        function  AppendWriteRegister (x : byte; data : word) : ansistring; // ein register-zugriff zu einem nibble-buffer umsetzen

        procedure WriteRegister (x : byte ; data : word; reset : boolean);  // internes register setzen und bertragen
                                                                            // bei reset = true wird der codec resettet und alle register bertragen
        function  GetRegister (reg : byte) : word;     // holt den wert des intern gesetzen registers

        procedure WriteRegisterwoR (x : byte ; data : word); // schreibt register OHNE Reset (wo = without)
        procedure WriteRegisterwR  (x : byte ; data : word); // schreibt register MIT Reset (w = with)

      public

        constructor Create (fpgacom : TFPGAComBase; chip : byte);  // codec nummer und verbindung zur rs232 mssen gegeben sein
        destructor Destroy; override;

        procedure ResetCodec;          // codec resetten

        property   CodecRegister [x : byte] : word read GetRegister write WriteRegisterwR; // register zugriffe (leider jedesmal mit reset)
      end;

implementation


//////////////////////////////////////////////////////////////////////////////
//
//  Mit dieser Klasse wird der Nibble Buffer fr die WriteGPO Funktionen der FPGACOMBase
//  Klasse aufgebaut. Die GPO-Leitungen konfigurieren die Codecs.
//  Diese Klasse stellt lediglich eine Low-level SPI-Klasse zur Verfgung
//  mit der die 4 Leitungen (2 Chip-Select, SPI-Clock und SPI-Data) gesetzt bzw. gelscht
//  werden knnen. Die eigentliche SPI-Funktionalitt wird in spteren Klassen gemacht.
//
//////////////////////////////////////////////////////////////////////////////


// initialisierung

constructor TCodecInterfaceBuilder.create;
begin
  temp      := '';
  cs1       := 1; cs2       := 1;
  spi_clk   := 0; spi_dout  := 0;
end;

// aktuellen port-status an string anhngen

procedure TCodecInterfaceBuilder.Append;
begin
  temp := temp + chr ( (cs2 shl 3) or (cs1 shl 2) or (spi_dout shl 1) or (spi_clk shl 0) );
end;

// neuen string zusammenbauen

procedure TCodecInterfaceBuilder.NewString;
begin
  temp := '';
end;

// setzen/lschen der 4 gpo-leitungen

procedure TCodecInterfaceBuilder.Nop;         begin                 Append; end;
procedure TCodecInterfaceBuilder.SetCS1;      begin cs1 := 0;       Append; end;
procedure TCodecInterfaceBuilder.ClrCS1;      begin cs1 := 1;       Append; end;
procedure TCodecInterfaceBuilder.SetCS2;      begin cs2 := 0;       Append; end;
procedure TCodecInterfaceBuilder.ClrCS2;      begin cs2 := 1;       Append; end;
procedure TCodecInterfaceBuilder.SetSPIClk;   begin spi_clk := 1;   Append; end;
procedure TCodecInterfaceBuilder.ClrSPIClk;   begin spi_clk := 0;   Append; end;
procedure TCodecInterfaceBuilder.SetSPIDout;  begin spi_dout := 1;  Append; end;
procedure TCodecInterfaceBuilder.ClrSPIDout;  begin spi_dout := 0;  Append; end;

// aktuell aufgebauten string zurckliefern und internen reset vollfhren

function  TCodecInterfaceBuilder.GetString : ansistring;
begin
  result := temp; temp := '';
  cs1       := 1; cs2       := 1;
  spi_clk   := 0; spi_dout  := 0;
end;






//////////////////////////////////////////////////////////////////////////////
//
// Diese Klasse kapselt das schreiben von SPI-Bytes per GPO via FPGA,
// erlaubt das schreiben von Registern beider Codecs, hlt die internen Register
// des Codecs vor (der TLV320AIC23B kann nur beschrieben werden) und kmmert
// sich um den Reset der Codecs. Dabei wird in abhngigkeit der Chip-Nummer
// wahlweise der Codec als Master bzw. Slave behandelt
//
//////////////////////////////////////////////////////////////////////////////


// Erstellt die CodecInterfaceKlasse, resettet die internen Register sowie den Codec
// mit der Chip-Nummer "chip"

constructor TCodecInterface.Create (fpgacom : TFPGAComBase; chip : byte);
begin
  cib       := TCodecInterfaceBuilder.create;
  ffpgacom  := fpgacom;

  // wir haben momentan nur 2 chips ...
  if (chip <= 1) then fchip := chip else fchip := 1;

  // register initialisieren
  InitData;

  // codec resetten
  ResetCodec;
end;

destructor TCodecInterface.Destroy;
begin
  cib.destroy;
end;

// erzeugt einen SPI-GPO-String fr den Zugriff auf ein Register des Codecs der
// durch die Chip-Nummer gegeben ist

function TCodecInterface.AppendWriteRegister (x : byte; data : word) : ansistring;
var w : word; c : byte;
begin
  // chip-nummer begrenzen
  fchip := max (0, min (1, fchip));
  // wir haben nur 16 register (eigentlich ja nur 11 ...)
  x   := x and $f;
  // 16-bit wort zusammenbauen
  w := (word (x) shl 9) or (data and $1ff);
  cib.NewString;

  // in abhngigkeit des chips mu CS1 oder CS 2 gesetzt werden
  if fchip = 0 then cib.SetCS1 else cib.SetCS2;

  // 16 bits rausclocken
  for c := 0 to 15 do
  begin
    cib.ClrSPIClk;
    if (w and $8000) <> 0 then cib.SetSPIDout
                          else cib.ClrSPIDout;
    cib.SetSPIClk;
    w := w shl 1;
  end;

  // alles deselektieren
  cib.ClrSPIClk;
  cib.ClrCS1;
  cib.ClrCS2;
  cib.Nop;

  // fertigen string zurckliefern
  result := cib.GetString;
end;


// schreibzugriff auf internes register (einzelne bertragung oder bertragung aller register mit reset)
procedure TCodecInterface.WriteRegister (x : byte; data : word; reset : boolean);
begin
  // wir knnen nur die ersten 10 register beschreiben (register 15 ist reset)
  if (ffpgacom <> NIL) and (x <= 9) then
  begin
    fregs [x] := data and $1ff;
    // alles senden (mit reset)
    if reset then SendAll
    // einzelnes register senden
             else ffpgacom.WriteGPOBuffer (AppendWriteRegister (x, data));
  end;
end;

// schreibzugriff (einzeln)
procedure TCodecInterface.WriteRegisterwoR (x : byte ; data : word);
begin
  WriteRegister (x, data, false);
end;

// schreibzugriff (mit reset und komplettbertragung)
procedure TCodecInterface.WriteRegisterwR  (x : byte ; data : word);
begin
  WriteRegister (x, data, true);
end;


// interne register anhand der chip-nummer vorbesetzen
procedure TCodecInterface.InitData;
begin
  if fchip = 0  then fregs [$07] := $004a      // master (20bit i2s no lr swap)
                else fregs [$07] := $000a;     // slave  (20bit i2s no lr swap)

  fregs [$00] := $0117;  // level l line in 0dB
  fregs [$01] := $0117;  // level r line in 0dB
  fregs [$02] := $01d5;  // hp-level l auf -36 dB (0dB ist SEHR laut)
  fregs [$03] := $01d5;  // hp-level r auf -36 dB (0dB ist SEHR laut)
  fregs [$04] := $0012;  // audio-path (dac ein, bypass aus line-select,, kein mic-boost kein side-tone)
  fregs [$05] := $0001;  // digital-path (kein softmute, kein adc-hp kein deemphasis)
  fregs [$06] := $0000;  // power off = aus (also alles an : adc, dac, mic-amp & & & ...)

  fregs [$08] := $0000;  // sample rate ctrl (kein mclk div, 48kHz, 256fs normal clock)
  fregs [$09] := $0001;  // digitales interface einschalten
end;

procedure TCodecInterface.SendAll;
var x : ansistring;
    c : integer;
begin
  if ffpgacom <> NIL then
  begin
    // erst codec resetten
    x := AppendWriteRegister ($0f, $0000);
    // alle register anhngen
    for c := 0 to 9 do
      x := x + AppendWriteRegister (c, fregs [c]);
    // und alles in einem rutsch raus
    ffpgacom.WriteGPOBuffer (x);
  end;
end;


// register initialisieren und alles senden
procedure TCodecInterface.ResetCodec;
begin
  InitData;
  SendAll;
end;


// internes register auslesen
function  TCodecInterface.GetRegister (reg : byte) : word;
begin
  result := 0;
  if reg <= 9 then
    result := fregs [reg];
end;



end.
