////////////////////////////////////////////////////////////////////////////
//
//  COEFFS.PAS
//
//  Hier werden "lose" Routinen zur Berchnung von Filterkoeffizienten zur
//  Verfgung gestellt. Die Koeffizienten basieren auf dem Buch
//  Digitale Audiosignalverarbeitung von Udo Zlzer (S. 137)
//  ISBN-10: 351916180X
//  Die Koeffizienten fr Bandpass und Bandsperre sind auf meinem "mist" gewachsen 
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007 (in der Form, ursprnglicher Quellcode von 2002)
//
//	(c) 2002-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit coeffs;

interface

uses windows, math,
     fpga_dap;

// Konstanten fr Halbtonschritte
const sfSemiTone   : single = 1.0594630943592952645618252949463;
const refSemiToneA : single = 57.0; // (MIDI-referenz note 57 = a')



function  p2f (n : single) : single;

// berechnet psse (tiefpass, hochpass, bandpass, notch, peak-filter, bypass, volume, mute)
procedure calccoeffLP (p, q : single; var x : TDAPBiquad);
procedure calccoeffHP (p, q : single; var x : TDAPBiquad);
procedure calccoeffBP (p, q : single; var x : TDAPBiquad);
procedure calccoeffBS (p, q : single; var x : TDAPBiquad);
procedure calccoeffPK (f, q, g  : single; var x : TDAPBiquad);
procedure calccoeffBypass (var x : TDAPBiquad);
procedure calccoeffMute   (var x : TDAPBiquad);
procedure calccoeffVolume (dB : single; var x : TDAPBiquad);


implementation

// rechnet halbton in frequenz um
function p2f (n : single) : single;
begin
  result := 440 * power (sfSemiTone, n - refSemiToneA);
end;

// tiefpass-filter berechnen
procedure calccoeffLP (p, q : single; var x : TDAPBiquad);
var fo, n, k, kq, qr : single;
begin
  fo    := p2f (min (120, p));
  k     := tan (pi * fo / 48000);  
  kq    := k * k;
  q     := max (-2, min (2, 2*q));
  qr    := sqrt (power (10, -q));

  n     := (1 + qr * k + kq);

  x.ulB [0] := trunc ($01000000 * (kq / n));
  x.ulB [1] := trunc ($01000000 * (2 * kq / n));
  x.ulB [2] := trunc ($01000000 * (kq / n));
  x.ulA [0] := round ($01000000 * (-2 * (kq - 1)        / n));
  x.ulA [1] := round ($01000000 * ((-(1 - qr * k + kq)) / n));
end;


// hochpass-filter berechnen
procedure calccoeffHP (p, q : single; var x : TDAPBiquad);
var fo, n, k, kq, qr : single;
begin
  fo    := p2f (min (120, p));
  k     := tan (pi * fo / 48000);
  kq    := k * k;
  q     := max (-2, min (2, 2*q));
  qr    := sqrt (2 * power (10, -q));

  n     := (1 + qr * k + kq);

  x.ulB [0] := round ($01000000 * ((1  / n)));
  x.ulB [1] := round ($01000000 * (-2 * (1  / n)));
  x.ulB [2] := round ($01000000 * ((1  / n)));
  x.ulA [0] := round ($01000000 * (-(2 * (kq - 1))    / n));
  x.ulA [1] := round ($01000000 * (-(1 - qr * k + kq) / n));
end;

// bandpass-filter berechnen
procedure calccoeffBP (p, q : single; var x : TDAPBiquad);
var fo, n, k, kq, qr : single;
begin
  fo    := p2f (min (120, p));
  k     := tan (pi * fo / 48000);
  kq    := k * k;
  q     := max (-2, min (2, 2*q));
  qr    := sqrt (power (10, -q));

  n     := (1 + qr * k + kq);

  x.ulB [0] := round ($01000000 * (((Qr * K + Kq)  / n)));
  x.ulB [1] :=        $00000000;
  x.ulB [2] := round ($01000000 * ((-(Qr * K + Kq)  / n)));
  x.ulA [0] := round ($01000000 * (-(2 * (kq - 1))    / n));
  x.ulA [1] := round ($01000000 * (-(1 - qr * k + kq) / n));
end;

// bandsperre-filter berechnen
procedure calccoeffBS (p, q : single; var x : TDAPBiquad);
var fo, n, k, kq, qr : single;
begin
  fo    := p2f (min (120, p));
  k     := tan (pi * fo / 48000);
  kq    := k * k;
  q     := max (-2, min (2, 2*q));
  qr    := sqrt (power (10, -q));

  n     := (1 + qr * k + kq);

  x.ulB [0] := round ($01000000 * ((1 + Kq)  / n));
  x.ulB [1] := round ($01000000 * (2.0 * (Kq - 1.0) ) / n);
  x.ulB [2] := round ($01000000 * ((1 + Kq)  / n));
  x.ulA [0] := round ($01000000 * (-(2 * (kq - 1))    / n));
  x.ulA [1] := round ($01000000 * (-(1 - qr * k + kq) / n));
end;

// peak-filter berechnen
procedure calccoeffPK (f, q, g  : single; var x : TDAPBiquad);
var fo, k, kq, n, v0 : single;
begin
  fo := max (0    , min (48000 * 0.45 , f));
  q  := max (0.01 , min (10           , q));
  g  := max (-12  , min (12           , g));

  k  := tan (pi * fo / 48000);
  kq := k * k;

  v0 := power (10, g / 20);

  if v0 > 1.0 then
    begin
      n := 1 + (1 / q) *k + kq;

      x.ulB [0] := round ($01000000 * ( (1 + (v0 / q)*k + kq)   / n));
      x.ulB [1] := round ($01000000 * ( 2 * (kq - 1)            / n));
      x.ulB [2] := round ($01000000 * ( (1 - (v0 / q)*k + kq)   / n));
      x.ulA [0] := round ($01000000 * (- (2 * (kq - 1))         / n));
      x.ulA [1] := round ($01000000 * (- (1 - (1 /q) * k + kq)  / n));
    end
  else if v0 = 1.0 then
    begin
      x.ulB [0] := round ($01000000 * 1.0);
      x.ulB [1] := round ($01000000 * 0.0);
      x.ulB [2] := round ($01000000 * 0.0);
      x.ulA [0] := round ($01000000 * 0.0);
      x.ulA [1] := round ($01000000 * 0.0);
    end
  else if v0 < 1.0 then
    begin
      v0 := 1 / v0;
      n := 1 + (v0 / q) *k + kq;

      x.ulB [0] := round ($01000000 * ( (1 + (1 / q)*k + kq)   / n));
      x.ulB [1] := round ($01000000 * (  2 * (kq - 1)          / n));
      x.ulB [2] := round ($01000000 * ( (1 - (1 / q)*k + kq)   / n));
      x.ulA [0] := round ($01000000 * (-(2 * (kq - 1))         / n));
      x.ulA [1] := round ($01000000 * (-(1 - (v0 /q) * k + kq) / n));
    end;
end;

// mute koeffizienten
procedure calccoeffMute (var x : TDAPBiquad);
begin
  x.ulB [0] := $00000000;
  x.ulB [1] := $00000000;
  x.ulB [2] := $00000000;
  x.ulA [0] := $00000000;
  x.ulA [1] := $00000000;
end;

// bypass koeffizienten
procedure calccoeffBypass (var x : TDAPBiquad);
begin
  x.ulB [0] := $01000000;
  x.ulB [1] := $00000000;
  x.ulB [2] := $00000000;
  x.ulA [0] := $00000000;
  x.ulA [1] := $00000000;
end;

// lautstrke koeffizienten
procedure calccoeffVolume (dB : single; var x : TDAPBiquad);
begin
  dB := max (-60, min (12, db));
  x.ulB [0] := round ($01000000 * power (10, db / 20));
  x.ulB [1] := $00000000;
  x.ulB [2] := $00000000;
  x.ulA [0] := $00000000;
  x.ulA [1] := $00000000;
end;


end.
