////////////////////////////////////////////////////////////////////////////
//
//  FPGA_COM_BASE.PAS
//
//  Stellt die Verbindung zum FPGA her (TFPGACOMBase kapselung von TSimple232)
//  und implementiert die Basis-Klasse fr FPGA-Gerte (z.B. DAP, StdIO, VGA-Text)
//
//  Hinweis : Bei RS232 bertragungen wird fast ausschlielich der Datentyp
//            "ansistring" verwendet (aufbauen [ohne getmem o.] und abschicken)
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////

unit fpga_com_base;

interface

uses  windows, classes, sysutils, stdctrls,

      simple232;


const fpgacom_baudrate : integer = 115200;   // standard baudrate fr fpga-kommuniktaion 

type // Einfache Kapselung der RS232-Schnittstelle (im Moment nur Schreibzugriffe)
     TFPGACOMBase = class
     private
       rs232 : TSimple232;    // RS232-Schnittstelle

     public
       constructor Create;
       destructor destroy; override;

       function  OpenPort (port : byte) : boolean;  // ffne COM Port
       procedure ClosePort;                         // schliee COM Port

       procedure vSendData (x : ansistring);        // fpga-rohdaten per rs232 versenden

       // Funktionen um GPO's zu setzen (fr Codec Inititialisierung)
       procedure SetGPO (ucNibble : byte);                    // gpo setzen (einzeln)
       procedure WriteGPOBuffer (nibblebuffer : ansistring);  // gpo nibble buffer schreiben
     end;

     //
     // Basis Klasse eines Gertes im FPGA welches per Data-Dispatcher
     // angesprochen werden kann
     //
     TFPGADevice = class
     protected
       fpgacom    : TFPGACOMBase;
       deviceport : char;

       // port des FPGA-Gertes
       procedure setport (x : byte);
       function  getport : byte;

       // daten senden (esc dabei ersetzen durch esc+esc)
       procedure SendData (x : ansistring);

       // fpga (datendispatcher) selektieren
       procedure SelectFPGA;
       // fpga (datendispatcher) deselektieren
       procedure DeselectFPGA;

     public

       constructor Create (base : TFPGACOMBase);
       destructor  Destroy; override;

       // einen 16-bit wert mit MSB First senden
       procedure Write16BitMSB (x : word);

       // port des Gertes im FPGA bestimmen
       property Port : byte read getport write setport;
     end;


// datentypen an ansistring anhngen

procedure AppendULONG (var x : ansistring; val : longint); forward;
procedure AppendUINT  (var x : ansistring; val : word);    forward;
procedure AppendUCHAR (var x : ansistring; val : byte);    forward;


// hexadezimal konvertierungen

function Byte2Hex (x : byte)    : ansistring; forward;
function Word2Hex (x : word)    : ansistring; forward;
function Long2Hex (x : longint) : ansistring; forward;

implementation

// fpga befehle als konstanten

const cFPGA_Select        : string = #$1b#$00;
const cFPGA_Deselect      : string = #$1b#$01;

// datentypen an ansistring anhngen


// longint binr (lsb) an string anhngen
procedure AppendULONG (var x : ansistring; val : longint);
begin
  x := x +  chr ((val shr  0) and $ff) + chr ((val shr  8) and $ff) +
            chr ((val shr 16) and $ff) + chr ((val shr 24) and $ff);
end;

// word binr (lsb) an string anhngen
procedure AppendUINT (var x : ansistring; val : word);
begin
  x := x +  chr ((val shr  0) and $ff) + chr ((val shr  8) and $ff);
end;

// byte binr an string anhngen
procedure AppendUCHAR (var x : ansistring; val : byte);
begin
  x := x +  chr (val);
end;




function byte2hex (x : byte) : ansistring;
var n : byte;
begin
  result := '';
  n := x shr 4;
  if n > 9  then result := result + chr (n - 10 + $41)
            else result := result + chr (n      + $30);
  n := x and $f;
  if n > 9  then result := result + chr (n - 10 + $41)
            else result := result + chr (n      + $30);
end;

function word2hex (x : word) : ansistring;
begin
  result := byte2hex (hi (x)) + byte2hex (lo (x));
end;

function long2hex (x : longint) : ansistring;
begin
  result := word2hex ((x shr 16) and $ffff) + word2hex ((x shr 0) and $ffff);
end;





//////////////////////////////////////////////////////////
//
//  TFPGACOMBase - Basis Klasse fr FPGA-Zugriff
//
//  Diese Klasse kapselt (zur Zeit) lediglich die RS232
//  Komponente (TSimple232). Spter werden dort "Empfnger"
//  eingetragen die bei Ankunft einer Nachricht eine Funktion
//  aufrufen (einfach). Somit werden in dieser Klasse vorerst nur
//  Sende-Routinen implementiert.
//
//////////////////////////////////////////////////////////

constructor TFPGACOMBase.Create;
begin
  // wir erzeugen eine rs232 schnitstelle
  rs232 := TSimple232.Create;
end;

function TFPGACOMBase.OpenPort (port : byte) : boolean;
begin
  // falls ein com-port geffnet ist -> schlieen
  if rs232.bComOk then rs232.ClosePort;
  // neuen com port aufmachen
  result := rs232.OpenPort(port, fpgacom_baudrate);
end;

procedure TFPGACOMBase.ClosePort;
begin
  rs232.ClosePort;
end;

destructor TFPGACOMBase.destroy;
begin
  rs232.ClosePort;
  rs232.Destroy;
end;

// roh-daten an fpga senden
procedure TFPGACOMBase.vSendData (x : ansistring);
begin
  // einfach senden ...
  rs232.vSendData(x);
end;

// gpo setzen
procedure TFPGACOMBase.SetGPO (ucNibble : byte);
begin
  // nur 4 bit
  ucNibble := ucNibble and $0f;
  // sequenz zusammenbauen und senden
  vSendData (#$1b + chr ($80 + ucNibble) + #$04);
end;


procedure TFPGACOMBase.WriteGPOBuffer (nibblebuffer : ansistring);
var send  : ansistring;
    c     : integer;
begin
  if nibblebuffer <> '' then
  begin
    // string aus nibbles zusammenbauen (bei jedem zeichen das 7. bit setzen)
    send := '';
    for c := 0 to length (nibblebuffer) - 1 do
      send := send + chr ($80 + (ord (nibblebuffer [c]) and $f));
    // sequenz senden
    vSendData (#$1b + send + #$04);
  end;
end;






//////////////////////////////////////////////////////////
//
//  TFPGADevice - Basis Klasse fr FPGA Gerte
//
//  Hier werden Grundlegende Funktionen zur Anstuerung des
//  FPGAs bereitgestellt. Mit Ableitungen dieser Klasse lsst
//  sich die Funktionalitt der einzelnen Komponenten im FPGA
//  nutzen. Es werden folgende Funktionen bereitgestellt :
//
//
//
//
//////////////////////////////////////////////////////////


//
// Standard
//

constructor TFPGADevice.Create (base : TFPGACOMBase);
begin
  inherited Create;
  // kommunikations klasse bernehmen
  fpgacom := base;
  // device-port lschen (MU PER PROPERTY PORT GESETZT WERDEN !!)
  deviceport := #0;
end;

destructor  TFPGADevice.Destroy;
begin
  inherited;
end;

//
// Port Funktionen
//

procedure TFPGADevice.setport (x : byte);
begin
  // nach initialisierung nur eine zuweisung zulassen
  if deviceport = #0 then
    deviceport := chr (x);
end;

function  TFPGADevice.getport : byte;
begin
  // device-port zurckgeben
  result := ord (deviceport);
end;

//
// FPGA Haupt-Funktionen
//


// Daten-String aufbereiten und senden
procedure TFPGADevice.SendData (x : ansistring);
var y : ansistring; c : integer;
begin
  // eigentich nur zeichen umkopieren ...
  y := '';
  if x > '' then
    for c := 1 to length (x) do
      // ... bei einem escape mssen wir dieses doppelt senden (fr andere esc-codes)
      if x [c] = #$1b then y := y + #$1b#$1b else y := y + x [c];
  fpgacom.vSendData(y);
end;

// FPGA Selektieren
procedure TFPGADevice.SelectFPGA;
begin
  // Select-Sequenz senden
  fpgacom.vSendData (cFPGA_Select);
end;

// FPGA Deselektieren
procedure TFPGADevice.DeselectFPGA;
begin
  // Deselect-Sequenz senden
  fpgacom.vSendData (cFPGA_Deselect);
end;

// 16 Bit Wert mit MSB First senden (fr Adressen z.b.)
procedure TFPGADevice.Write16BitMSB (x : word);
begin
  SendData (chr (hi (x)) + chr (lo (x)));
end;





end.
